@@ -7,6 +7,7 @@ pub mod loader;
 pub const FONT_ID_REGULAR: usize = 0;
 
 use crate::font::constants::*;
+use crate::font::fonts::{parse_unicode, SugarloafFontStyle, SugarloafFontWidth};
 use crate::font_introspector::text::cluster::Parser;
 use crate::font_introspector::text::cluster::Token;
 use crate::font_introspector::text::cluster::{CharCluster, Status};
@@ -15,11 +16,9 @@ use crate::font_introspector::text::Script;
 use crate::font_introspector::{CacheKey, FontRef, Synthesis};
 use crate::layout::FragmentStyle;
 use crate::SugarloafErrors;
-use ab_glyph::FontArc;
-use lru::LruCache;
-use parking_lot::FairMutex;
+use parking_lot::RwLock;
 use rustc_hash::FxHashMap;
-use std::num::NonZeroUsize;
+use std::ops::Range;
 use std::path::PathBuf;
 use std::sync::Arc;
 
@@ -28,13 +27,13 @@ pub use crate::font_introspector::{Style, Weight};
 pub fn lookup_for_font_match(
     cluster: &mut CharCluster,
     synth: &mut Synthesis,
-    library: &mut FontLibraryData,
+    library: &FontLibraryData,
     spec_font_attr_opt: Option<&(crate::font_introspector::Style, bool)>,
 ) -> Option<(usize, bool)> {
     let mut search_result = None;
     let mut font_synth = Synthesis::default();
-    let fonts_len: usize = library.inner.len();
 
+    let fonts_len: usize = library.inner.len();
     for font_id in 0..fonts_len {
         let mut is_emoji = false;
 
@@ -45,22 +44,29 @@ pub fn lookup_for_font_match(
             // In this case, the font does match however
             // we need to check if is indeed a match
             if let Some(spec_font_attr) = spec_font_attr_opt {
-                if font.style != spec_font_attr.0 {
+                let style_is_different = font.style != spec_font_attr.0;
+                let is_italic = spec_font_attr.0 == Style::Italic;
+                if style_is_different && is_italic && !font.should_italicize {
                     continue;
                 }
 
                 // In case bold is required
                 // It follows spec on Bold (>=700)
                 // https://developer.mozilla.org/en-US/docs/Web/CSS/@font-face/font-weight
-                if spec_font_attr.1 && font.weight < crate::font_introspector::Weight(700)
-                {
+                let weight_is_different = spec_font_attr.1 && font.weight < Weight(700);
+                if weight_is_different && !font.should_embolden {
                     continue;
                 }
             }
         }
 
-        if let Some(data) = library.get_data(&font_id) {
-            let charmap = data.charmap();
+        if let Some((shared_data, offset, key)) = library.get_data(&font_id) {
+            let font_ref = FontRef {
+                data: shared_data.as_ref(),
+                offset,
+                key,
+            };
+            let charmap = font_ref.charmap();
             let status = cluster.map(|ch| charmap.map(ch));
             if status != Status::Discard {
                 *synth = font_synth;
@@ -81,7 +87,7 @@ pub fn lookup_for_font_match(
 
 #[derive(Clone)]
 pub struct FontLibrary {
-    pub inner: Arc<FairMutex<FontLibraryData>>,
+    pub inner: Arc<RwLock<FontLibraryData>>,
 }
 
 impl FontLibrary {
@@ -97,7 +103,7 @@ impl FontLibrary {
 
         (
             Self {
-                inner: Arc::new(FairMutex::new(font_library)),
+                inner: Arc::new(RwLock::new(font_library)),
             },
             sugarloaf_errors,
         )
@@ -110,24 +116,29 @@ impl Default for FontLibrary {
         let _fonts_not_found = font_library.load(SugarloafFonts::default());
 
         Self {
-            inner: Arc::new(FairMutex::new(font_library)),
+            inner: Arc::new(RwLock::new(font_library)),
         }
     }
 }
 
+pub struct SymbolMap {
+    pub font_index: usize,
+    pub range: Range<char>,
+}
+
 pub struct FontLibraryData {
-    pub ui: FontArc,
     // Standard is fallback for everything, it is also the inner number 0
     pub inner: FxHashMap<usize, FontData>,
-    pub stash: LruCache<usize, SharedData>,
+    pub symbol_maps: Option<Vec<SymbolMap>>,
+    pub hinting: bool,
 }
 
 impl Default for FontLibraryData {
     fn default() -> Self {
         Self {
-            ui: FontArc::try_from_slice(FONT_CASCADIAMONO_REGULAR).unwrap(),
             inner: FxHashMap::default(),
-            stash: LruCache::new(NonZeroUsize::new(2).unwrap()),
+            hinting: true,
+            symbol_maps: None,
         }
     }
 }
@@ -135,7 +146,7 @@ impl Default for FontLibraryData {
 impl FontLibraryData {
     #[inline]
     pub fn find_best_font_match(
-        &mut self,
+        &self,
         ch: char,
         fragment_style: &FragmentStyle,
     ) -> Option<(usize, bool)> {
@@ -155,6 +166,15 @@ impl FontLibraryData {
             return Some((0, false));
         }
 
+        // First check symbol map before lookup_for_font_match
+        if let Some(symbol_maps) = &self.symbol_maps {
+            for symbol_map in symbol_maps {
+                if symbol_map.range.contains(&ch) {
+                    return Some((symbol_map.font_index, false));
+                }
+            }
+        }
+
         let is_italic = fragment_style.font_attrs.style() == Style::Italic;
         let is_bold = fragment_style.font_attrs.weight() == Weight::BOLD;
 
@@ -186,39 +206,22 @@ impl FontLibraryData {
     }
 
     #[inline]
-    pub fn get(&mut self, font_id: &usize) -> &FontData {
+    pub fn get(&self, font_id: &usize) -> &FontData {
         &self.inner[font_id]
     }
 
-    pub fn get_data<'a>(&'a mut self, font_id: &usize) -> Option<FontRef<'a>> {
+    pub fn get_data(&self, font_id: &usize) -> Option<(SharedData, u32, CacheKey)> {
         if let Some(font) = self.inner.get(font_id) {
-            match &font.data {
-                Some(data) => {
-                    return Some(FontRef {
-                        data: data.as_ref(),
-                        offset: font.offset,
-                        key: font.key,
-                    })
-                }
-                None => {
-                    if !self.stash.contains(font_id) {
-                        if let Some(path) = &font.path {
-                            if let Some(raw_data) = load_from_font_source(path) {
-                                self.stash.put(*font_id, SharedData::new(raw_data));
-                            }
-                        }
-                    }
+            if let Some(data) = &font.data {
+                return Some((data.clone(), font.offset, font.key));
+            } else if let Some(path) = &font.path {
+                // Load font data directly from file when needed
+                if let Some(raw_data) = load_from_font_source(path) {
+                    let shared_data = SharedData::new(raw_data);
+                    return Some((shared_data, font.offset, font.key));
                 }
             }
-
-            if let Some(data) = self.stash.get(font_id) {
-                return Some(FontRef {
-                    data: data.as_ref(),
-                    offset: font.offset,
-                    key: font.key,
-                });
-            }
-        };
+        }
 
         None
     }
@@ -240,6 +243,9 @@ impl FontLibraryData {
 
     #[cfg(not(target_arch = "wasm32"))]
     pub fn load(&mut self, mut spec: SugarloafFonts) -> Vec<SugarloafFont> {
+        // Configure hinting through spec
+        self.hinting = spec.hinting;
+
         let mut fonts_not_fount: Vec<SugarloafFont> = vec![];
 
         // If fonts.family does exist it will overwrite all families
@@ -253,15 +259,23 @@ impl FontLibraryData {
         let mut db = loader::Database::new();
         db.load_system_fonts();
 
+        spec.additional_dirs
+            .unwrap_or_default()
+            .into_iter()
+            .map(PathBuf::from)
+            .for_each(|p| db.load_fonts_dir(p));
+
         match find_font(&db, spec.regular, false, false) {
             FindResult::Found(data) => {
                 self.insert(data);
             }
             FindResult::NotFound(spec) => {
-                self.insert(load_fallback_from_memory(&spec));
                 if !spec.is_default_family() {
-                    fonts_not_fount.push(spec);
+                    fonts_not_fount.push(spec.to_owned());
                 }
+
+                // The first font should always have a fallback
+                self.insert(load_fallback_from_memory(&spec));
             }
         }
 
@@ -270,9 +284,10 @@ impl FontLibraryData {
                 self.insert(data);
             }
             FindResult::NotFound(spec) => {
-                self.insert(load_fallback_from_memory(&spec));
                 if !spec.is_default_family() {
                     fonts_not_fount.push(spec);
+                } else {
+                    self.insert(load_fallback_from_memory(&spec));
                 }
             }
         }
@@ -282,9 +297,10 @@ impl FontLibraryData {
                 self.insert(data);
             }
             FindResult::NotFound(spec) => {
-                self.insert(load_fallback_from_memory(&spec));
                 if !spec.is_default_family() {
                     fonts_not_fount.push(spec);
+                } else {
+                    self.insert(load_fallback_from_memory(&spec));
                 }
             }
         }
@@ -294,9 +310,10 @@ impl FontLibraryData {
                 self.insert(data);
             }
             FindResult::NotFound(spec) => {
-                self.insert(load_fallback_from_memory(&spec));
                 if !spec.is_default_family() {
                     fonts_not_fount.push(spec);
+                } else {
+                    self.insert(load_fallback_from_memory(&spec));
                 }
             }
         }
@@ -344,6 +361,7 @@ impl FontLibraryData {
                     family: extra_font.family,
                     style: extra_font.style,
                     weight: extra_font.weight,
+                    width: extra_font.width,
                 },
                 true,
                 true,
@@ -359,18 +377,67 @@ impl FontLibraryData {
 
         self.insert(FontData::from_slice(FONT_SYMBOLS_NERD_FONT_MONO, false).unwrap());
 
-        if let Some(ui_spec) = spec.ui {
-            match find_font(&db, ui_spec, false, false) {
-                FindResult::Found(data) => {
-                    self.ui = FontArc::try_from_vec(data.data.unwrap().to_vec()).unwrap();
-                }
-                FindResult::NotFound(spec) => {
-                    fonts_not_fount.push(spec);
+        // TODO: Currently, it will naively just extend fonts from symbol_map
+        // without even look if the font has been loaded before.
+        // Considering that we drop the font data that's inactive should be ok but
+        // it will cost a bit more time to initialize.
+        //
+        // Considering we receive via config
+        // [{ start = "2297", end = "2299", font-family = "Cascadia Code NF" },
+        //  { start = "2296", end = "2297", font-family = "Cascadia Code NF" }]
+        //
+        // Will become:
+        // [{ start = "2297", end = "2299", font_index = Some(1) },
+        //  { start = "2296", end = "2297", font_index = Some(1) }]
+        //
+        // TODO: We should have a new symbol map internally
+        // { range = '2296'..'2297', font_index = Some(1) }]
+        if let Some(symbol_map) = spec.symbol_map {
+            let mut symbol_maps = Vec::default();
+            for extra_font_from_symbol_map in symbol_map {
+                match find_font(
+                    &db,
+                    SugarloafFont {
+                        family: extra_font_from_symbol_map.font_family,
+                        ..SugarloafFont::default()
+                    },
+                    true,
+                    true,
+                ) {
+                    FindResult::Found(data) => {
+                        if let Some(start) =
+                            parse_unicode(&extra_font_from_symbol_map.start)
+                        {
+                            if let Some(end) =
+                                parse_unicode(&extra_font_from_symbol_map.end)
+                            {
+                                self.insert(data);
+
+                                symbol_maps.push(SymbolMap {
+                                    range: start..end,
+                                    font_index: self.len() - 1,
+                                });
+
+                                continue;
+                            }
+                        }
+
+                        warn!("symbol-map: Failed to parse start and end values");
+                    }
+                    FindResult::NotFound(spec) => {
+                        fonts_not_fount.push(spec);
+                    }
                 }
             }
+
+            self.symbol_maps = Some(symbol_maps);
         }
 
-        fonts_not_fount
+        if spec.disable_warnings_not_found {
+            vec![]
+        } else {
+            fonts_not_fount
+        }
     }
 
     #[cfg(target_arch = "wasm32")]
@@ -424,6 +491,8 @@ pub struct FontData {
     pub style: crate::font_introspector::Style,
     pub stretch: crate::font_introspector::Stretch,
     pub synth: Synthesis,
+    pub should_embolden: bool,
+    pub should_italicize: bool,
     pub is_emoji: bool,
 }
 
@@ -442,6 +511,7 @@ impl FontData {
         path: PathBuf,
         evictable: bool,
         is_emoji: bool,
+        font_spec: &SugarloafFont,
     ) -> Result<Self, Box<dyn std::error::Error>> {
         let font = FontRef::from_index(&data, 0).unwrap();
         let (offset, key) = (font.offset, font.key);
@@ -451,6 +521,12 @@ impl FontData {
         let attributes = font.attributes();
         let style = attributes.style();
         let weight = attributes.weight();
+
+        let should_italicize =
+            font_spec.style == SugarloafFontStyle::Italic && style != Style::Italic;
+
+        let should_embolden = font_spec.weight >= Some(700) && weight < Weight(700);
+
         let stretch = attributes.stretch();
         let synth = attributes.synthesize(attributes);
 
@@ -463,6 +539,8 @@ impl FontData {
         Ok(Self {
             data,
             offset,
+            should_italicize,
+            should_embolden,
             key,
             synth,
             style,
@@ -494,6 +572,8 @@ impl FontData {
             key,
             synth,
             style,
+            should_embolden: false,
+            should_italicize: false,
             weight,
             stretch,
             path: None,
@@ -508,14 +588,6 @@ pub type SugarloafFonts = fonts::SugarloafFonts;
 #[cfg(not(target_arch = "wasm32"))]
 use tracing::{info, warn};
 
-#[derive(Debug, Clone)]
-pub struct ComposedFontArc {
-    pub regular: FontArc,
-    pub bold: FontArc,
-    pub italic: FontArc,
-    pub bold_italic: FontArc,
-}
-
 enum FindResult {
     Found(FontData),
     NotFound(SugarloafFont),
@@ -542,15 +614,37 @@ fn find_font(
             query.weight = crate::font::loader::Weight(weight);
         }
 
-        if let Some(ref style) = font_spec.style {
-            let query_style = match style.to_lowercase().as_str() {
-                "italic" => crate::font::loader::Style::Italic,
-                _ => crate::font::loader::Style::Normal,
+        if let Some(ref width) = font_spec.width {
+            query.stretch = match width {
+                SugarloafFontWidth::UltraCondensed => {
+                    crate::font::loader::Stretch::UltraCondensed
+                }
+                SugarloafFontWidth::ExtraCondensed => {
+                    crate::font::loader::Stretch::ExtraCondensed
+                }
+                SugarloafFontWidth::Condensed => crate::font::loader::Stretch::Condensed,
+                SugarloafFontWidth::SemiCondensed => {
+                    crate::font::loader::Stretch::SemiCondensed
+                }
+                SugarloafFontWidth::Normal => crate::font::loader::Stretch::Normal,
+                SugarloafFontWidth::SemiExpanded => {
+                    crate::font::loader::Stretch::SemiExpanded
+                }
+                SugarloafFontWidth::Expanded => crate::font::loader::Stretch::Expanded,
+                SugarloafFontWidth::ExtraExpanded => {
+                    crate::font::loader::Stretch::ExtraExpanded
+                }
+                SugarloafFontWidth::UltraExpanded => {
+                    crate::font::loader::Stretch::UltraExpanded
+                }
             };
-
-            query.style = query_style;
         }
 
+        query.style = match font_spec.style {
+            SugarloafFontStyle::Italic => crate::font::loader::Style::Italic,
+            _ => crate::font::loader::Style::Normal,
+        };
+
         info!("Font search: '{query:?}'");
 
         match db.query(&query) {
@@ -566,6 +660,7 @@ fn find_font(
                                 path.to_path_buf(),
                                 evictable,
                                 is_emoji,
+                                &font_spec,
                             ) {
                                 Ok(d) => {
                                     tracing::info!(
@@ -596,20 +691,28 @@ fn find_font(
 }
 
 fn load_fallback_from_memory(font_spec: &SugarloafFont) -> FontData {
-    let style = font_spec.style.to_owned().unwrap_or("regular".to_string());
+    let style = &font_spec.style;
     let weight = font_spec.weight.unwrap_or(400);
 
-    let font_to_load = match (weight, style.as_str()) {
-        (100, "italic") => constants::FONT_CASCADIAMONO_EXTRA_LIGHT_ITALIC,
-        (200, "italic") => constants::FONT_CASCADIAMONO_LIGHT_ITALIC,
-        (300, "italic") => constants::FONT_CASCADIAMONO_SEMI_LIGHT_ITALIC,
-        (400, "italic") => constants::FONT_CASCADIAMONO_ITALIC,
-        (500, "italic") => constants::FONT_CASCADIAMONO_ITALIC,
-        (600, "italic") => constants::FONT_CASCADIAMONO_SEMI_BOLD_ITALIC,
-        (700, "italic") => constants::FONT_CASCADIAMONO_SEMI_BOLD_ITALIC,
-        (800, "italic") => constants::FONT_CASCADIAMONO_BOLD_ITALIC,
-        (900, "italic") => constants::FONT_CASCADIAMONO_BOLD_ITALIC,
-        (_, "italic") => constants::FONT_CASCADIAMONO_ITALIC,
+    let font_to_load = match (weight, style) {
+        (100, SugarloafFontStyle::Italic) => {
+            constants::FONT_CASCADIAMONO_EXTRA_LIGHT_ITALIC
+        }
+        (200, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_LIGHT_ITALIC,
+        (300, SugarloafFontStyle::Italic) => {
+            constants::FONT_CASCADIAMONO_SEMI_LIGHT_ITALIC
+        }
+        (400, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_ITALIC,
+        (500, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_ITALIC,
+        (600, SugarloafFontStyle::Italic) => {
+            constants::FONT_CASCADIAMONO_SEMI_BOLD_ITALIC
+        }
+        (700, SugarloafFontStyle::Italic) => {
+            constants::FONT_CASCADIAMONO_SEMI_BOLD_ITALIC
+        }
+        (800, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_BOLD_ITALIC,
+        (900, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_BOLD_ITALIC,
+        (_, SugarloafFontStyle::Italic) => constants::FONT_CASCADIAMONO_ITALIC,
         (100, _) => constants::FONT_CASCADIAMONO_EXTRA_LIGHT,
         (200, _) => constants::FONT_CASCADIAMONO_LIGHT,
         (300, _) => constants::FONT_CASCADIAMONO_SEMI_LIGHT,
